%\% Copyright Graham Freeman, March 1994
%\% gfreeman@cs.adfa.oz.au
%PN%%PS%
QSdict /PagStr (Family Tree Drawing  ) put
PE
%NF%%CL%Computer Science Department
%CL%University College
%CL%Australian Defence Force Academy
%CL%Campbell, ACT 2600
%CL%Australia
%FI%%VM,20%
%SZ,16%%BD%%CL%GENEAL: A Family Tree Drawing System
%L%%SZ,12%%CL%T. Graham Freeman
%L,.5%%LT%%CL%Computer Science Department Report CS04/94
%L,.5%%CL%15 March 1994
%VM,15%
%CL%%BD%Abstract
%LT%%CW,140%%TB,25%%P%
A convention for digitally representing family tree information
is described, along with a novel format for drawing the tree.
The format is suitable for large families, is flexible in
allowing the tree to be easily updated, and provides for
biographical detail to be included.  A program has been developed in
PostScript to process such trees, the output of which is
demonstrated.  Finally, an indexing scheme for locating individuals in 
a tree is proposed.
%VM,10%%TB,0%%CW%%P%
People who have spent any time working with family trees usually
find themselves confronting the problem of how to represent the
large volume of information, and how to keep it up-to-date.  
Different types of trees are used, depending on the purpose of
drawing it up.
Family trees that only represent direct ancestry are sometimes
known as pedigree charts.  They can double in size with each older
generation added, but generally do not present a problem in
representing the information because the information is increasingly
difficult to obtain with each generation.  Trees that contain
information on aunts, uncles and their children increase in size
much more rapidly with each added generation, and it is this type of
tree that we will consider in this paper.
%P%
The most common representation is on a large sheet of paper, with the
oldest ancestors at the top, their children beneath, and so forth
down to the present generation.  The sheet must be large once the
family tree has progressed back through four or more generations,
particularly with the large family sizes of earlier this century 
and before.  The size of the sheet and the nature of the 
information presented
usually require that layout be done by hand, a job almost for a 
draftsman.
The representation of more than one ancestry on the tree is 
difficult.  And finally, when the tree has been drawn, it is usually
goes out of date in a short period because of births or deaths in the
current generation, or because verbally collected information is
found to be inaccurate, or because more information about a remote
branch of the family comes to light.
%P%
In looking at these problems, I have developed a scheme that
solves most of them.  It uses a computer representation of
family relationships, and a computer program to draw the tree
using the digitally stored data.  It adopts a novel format for
the tree that simplifies automatic production of drawings, 
using normal sheet size (eg. A4), so that the whole
tree is broken into separate pieces, while retaining a feel for
how one branch of the tree relates to others.  It allows comments
to be included within the page, including anecdotes and pieces of
family history, that give more life and interest to the document.
The tree is thus drawn on a set of separate sheets, which can
be individually replaced as required, and could be held in a loose-leaf
folder.
%L%%PH%%BD%
Document layout
%LT%%P%
Simply breaking a large tree into pieces, with one piece per page,
would make representation simple, but would make reading it very 
awkward.  The task of establishing how one person is related to
another, one of the main benefits of having a tree, would be tedious
to say the least.  To overcome this, the proposal is to divide each
page into two or possibly three sections.  
%P,10,-10,-.5%%C,183%%T,10%The first section, at the top, is
for representing the ancestry of the particular family branch.
It would not have detail other than how the people on this page
are related to their forbears.
%P%%C,183%%T,10%The central section, which I call the "generation", 
gives the 
particular family unit and their offspring, along with any details
of birth, death or marriage dates and other brief information
where it is known.
%P%%C,183%%T,10%The third optional section would contain 
comments on the family,
such as a brief history of the type of employment, addresses, or 
influence they may have had on the local community.  This section may
be large enough to spill onto the reverse side of the sheet and
possibly subsequent pages.  
%P,0,0,0%
The ancestry section could become quite large if there was interest
in following back paternal and maternal lineage for many generations.
However, with only the most basic information included and by use of a 
small font, sufficient space should be left on the page for at least 
the generation section, with the comments spilling over the page.
%P%
In drawing the tree or tree segment, a slightly different format is
proposed from that conventionally used.  The most common tree form
adopted is to place the oldest ancestor at the top, the children in
a horizontal line beneath, and successive generations in the
same manner.  This quickly presents a problem because there are
often too many names to fit horizontally.  When drawing by hand, the
solution is commonly to offset some names vertically to make them
fit, using linking lines to visually maintain the relationships.
%P%
If a computer program is to be used to draw the tree automatically,
such offsetting schemes are difficult to specify in a general way.
%P%
Instead, the scheme proposed is still to have the oldest ancestor
at the top, but to have the children in a vertical column beneath
and to the right, with the next generation again offset further to the 
right, and so on.  This scheme implies time running down the page
in the order of birth of children, and to the right for succeeding
generations, whereas the traditional form has reversed directional 
role, representing time with
order of birth of children running to the right and succeeding 
generations running down the page.  This is similar to Pedigree Charts,
except that in the proposed scheme, time runs forward as you move to the
right, and allows for details of siblings to be included
at each generation.
%P%
An example of the use of the scheme can be seen in Figure 1.  
When a person
has descendents shown on another sheet, the convention adopted here is
to mark that name with an asterisk.  This is not a formal requirement 
in the scheme I propose, but rather a convention to make the sheets
easier to follow.
%L%%PH%%BD%
Digital representation of a tree
%LT%%P%
The real benefits of the proposed format will only come about if
the tree can be stored electronically, so that it can be edited and
reprinted easily.  To this end, a format must be devised for
storing the information.
%P%
The most sophisticated approach to this would be to use a
database.  This has not been done in this case for two main reasons.  
Firstly, the main use to which the family tree data will be put is 
expected to be purely graphical output.  The printed tree, to be read by 
people requiring no computer expertise, is the dominant 
purpose of collecting and storing the information.
Secondly, the design of a database structure capable of handling
remarriages, illegitimacies and other time-changing relationships,
non-uniqueness of individual's names in providing access to the data,
and allowance for variable length comments on individuals as well as
families, is an intellectual challenge.  If such a database
can be designed, then the format proposed here would be a suitable
intermediate form for report output from the database, prior to
graphics generation.
%P%
Instead, a sequential file mirroring in structure the planned
graphical form of output has been chosen.
Within this file, the keywords "<ancestry>", "<generation>", and
"<comment>" at the beginning of a line indicate the start of the
major sections of a part of a tree to be placed on a page.  
%P%
Within the first two sections, the structure of the family is
indicated using the keywords shown below.  These keywords will be
the first text encountered on a line.  The keywords may 
be preceded by white space on the line, in order to make the file more
readable; this white space will be ignored during processing.
%P,40,-40,-.5%
<children>         %T,40%The names of one or more children by the most
recently specified marital relationship will follow.  They will be
graphically presented suitably indented from the parents.
%P%
<nomorechildren>   %T,40%This signifies the end of the list of children
in the most recently specified marital relationship.  Any children's
names that follow will continue the previously unterminated list of
children.
%P%
<>                 %T,40%This indicates that the name of a child in the 
current list of children follows on this line.  There may be embedded 
comments, this child may have married, and these details can be included 
in the line, as discussed below.
%P%
<*>                %T,40%This indicates a child's name follows as for
"<>".  It is to be highlighted in the graphical output.  If it
appears in the ancestry section, this child will be recorded as 
the ancestor of the next generation if another generation follows.
%TB,0%%P,0,0,0%%FN,Hel%
<ancestry>
Frederich Kuffner = Anna Margaretha Edinger
<children>
<>Charlie 
<>Adam Kuffner = Amelia Rosalia Kuhn
<>
<>Anna 
<>Peter 
<>Carl Frederick 
<>Katherine 
<>Leonard 
<>Mary 
	<children>
	<>Fred 
	<>Kate 
	<>George 
	<>Amelia Kuffner = Edmund Spence
	<>Mary 
	<>Margaratha Anna 
	<>Adam 
	<>Charlie 
	<>Teeny 
		<children>
		<>Edward 
		<>Annie 
		<*>Thomas Spence 
		<>Margaret 
		<>William 
		<>Michael Joseph 
		<>John 
		<>James 
		<>Bernard Neil 
		<nomorechildren>
	<nomorechildren>
<nomorechildren>
<ancestry>
Christian Kuhn = Anna Katherina Morenhaur
	<children>
	<>Amelia Kuhn
	<nomorechildren>
<generation>
	Thomas Spence [born 1909] =[11 Nov 1931] Nell Watts [born 1911//died 1985]
		<children>
		<>Betty Spence = Laurie Lamont
		   live in Charters Towers
			<children>
			<>Rosalyn Dawn Lamont = Daryl Shaffer
				<children>
				<>Darrin Shafer *
				<>Brendon Shafer
				<>Shannon Shafer
				<>Trent Shafer
				<nomorechildren>
			<>Rhonda Kay Lamont = John Smith
				<children>
				<>Laurel Ann Smith
				<>Sonia Kay Smith
				<nomorechildren>
			<nomorechildren>
		<>Graham Spence 
		   unmarried, living in Charters Towers
		<>Marjorie Spence = William Findley
		   2 children
		<>Norman Spence = Doreen ?
		  lived in Brisbane
			<children>
			<>Mark Spence = Mary ?
			<nomorechildren>
			<divorced>
		   ~ Kelly Keeler
			<children>
			<>Brett Spence
			<nomorechildren>
		<nomorechildren>
<comment>
William Findlay's father was a police sergeant in Toowoomba.  Thomas
helped Marjorie and John with the purchase of a farm
at Kingaroy.  The property was poor, full of stone, and it drove
Findlay mad.  When his mother-in-law (Nell) was visiting a
daughter in Charters Towers for her son-in-law's
funeral, Findlay shot his two
children, his wife Marjorie, and also old Thomas.  He set fire to the 
farmhouse before killing himself.  Thomas was in his seventies.

%FN,Tim%%SZ,12%%L%
%PS%
 gsave QSdict begin
   currentpoint exch 2 MM sub exch moveto 170 MM 0 rlineto
   currentpoint pop PagLen TopMrg sub 2 MM add lineto
   -170 MM 0 rlineto closepath 
   closepath
   1 setlinewidth stroke grestore 
PE
%L,.5%%BD%Figure 1%LT%: Example of the proposed family tree layout,
showing the three sections: ancestry, generation, and comment.%L,.5%%TB%
%P,40,-40,-.5%
<divorced>         %T,40%The previous marriage was formally dissolved.
A comment to this effect is needed in the graphical output, placed in
such a way as to clearly relate it to the marriage relationship.
%P%
<separated>        %T,40%The previous couple in a marital relationship
separated, and a carefully placed comment to this effect is needed 
in the graphical output.
%P%
=                  %T,40%The most recently mentioned family member
who was in a marital relationship was (re)married.  The name of the
new spouse would follow on the line.  This will be placed on the
page in such a way to clearly relate the marriage to the correct person.
%P%
~                  %T,40%The most recently mentioned family member
who was in a marital relationship entered into a new marital 
relationship
without formal legal recognition, or a relationship of unknown status.  
The name of the new partner would follow on the line.  This will be 
placed on the page in such a way to clearly relate the 
relationship to the correct person.
%P%
Anything else      %T,40%Lines that lack any of the above keywords
at the start of a line (following any white space)
are treated as comment.  Comments
will be placed on a new line, in a small font, and indented from the
previous person's name.
%P,0,0,0%
In processing a person's name, provision is made for that person's
marriage and spouse to be recorded, using "=" or "~" as above.  It is
also possible to record brief comments about the person, the marriage,
and the spouse, by enclosing such commentary between "[" and "]"
markers.  The comments will be placed on the drawn tree
beneath the text to which it refers.  If comments
are of sufficient length to spill over several lines, this will
be done automatically.  Specific line breaks can be indicated with 
the "//" characters.
%P%
The family details given in Figure 1 could be specified using this
scheme as shown below:
%NF%%FN,Cou%%SZ,9%%TB,5%%L%%W,5%
%C,60%ancestry>
Frederich Kuffner = Anna Margaretha Edinger
<children>
<>Charlie 
<>Adam Kuffner = Amelia Rosalia Kuhn
<>
<>Anna 
<>Peter 
<>Carl Frederick 
<>Katherine 
<>Leonard 
<>Mary 
        <children>
        <>Fred 
        <>Kate 
        <>George 
        <>Amelia Kuffner = Edmund Spence
        <>Mary 
        <>Margaratha Anna 
        <>Adam 
        <>Charlie 
        <>Teeny 
                <children>
                <>Edward 
                <>Annie 
                <*>Thomas Spence 
                <>Margaret 
                <>William 
                <>Michael Joseph 
                <>John 
                <>James 
                <>Bernard Neil 
                <nomorechildren>
        <nomorechildren>
<nomorechildren>
%C,60%ancestry>
Christian Kuhn = Anna Katherina Morenhaur
        <children>
        <>Amelia Kuhn
        <nomorechildren>
%C,60%generation>
Thomas Spence [born 1909] =[11 Nov 1931] Nell Watts [born 1911//died 1985]
        <children>
        <>Betty Spence = Laurie Lamont
           live in Charters Towers
                <children>
                <>Rosalyn Dawn Lamont = Daryl Shaffer
                        <children>
                        <>Darrin Shafer
                        <>Brendon Shafer
                        <>Shannon Shafer
                        <>Trent Shafer
                        <nomorechildren>
                <>Rhonda Kay Lamont = John Smith
                        <children>
                        <>Laurel Ann Smith
                        <>Sonia Kay Smith
                        <nomorechildren>
                <nomorechildren>
        <>Graham Spence 
           unmarried, living in Charters Towers
        <>Marjorie Spence = William Findley
           2 children
        <>Norman Spence = Doreen ?
          lived in Brisbane
                <children>
                <>Mark Spence = Mary ?
                <nomorechildren>
                <divorced>
             ~ Kelly Keeler
                <children>
                <>Brett Spence
                <nomorechildren>
        <nomorechildren>
<comment>
William Findlay's father was a police sergeant in Toowoomba.  Thomas
helped Marjorie and John with the purchase of a farm at Kingaroy.  The 
property was poor, full of stone, and it drove Findlay mad.  When his 
mother-in-law (Nell) was visiting a daughter in Charters Towers for 
her son-in-law's funeral, Findlay shot his two children, his wife 
Marjorie, and also old Thomas.  He set fire to the farmhouse before 
killing himself.  Thomas was in his seventies.

%FI%%FN%%SZ%%TB,0%%W,6%
%\% %PS%
%\%  gsave QSdict begin
%\%    Where 10 get Where 11 get moveto 
%\%    Where 12 get Where 13 get lineto 
%\%    160 MM 0 rlineto 
%\%    currentpoint pop Where 11 get lineto
%\%    closepath
%\%    1 setlinewidth stroke 
%\%  grestore 
%\% PE
%P%
In designing this specification scheme, the intent has been to
use terms and layout that will be comfortable for people with
relatively little computing experience, while having the precision
and absence of ambiguity that will be essential for successful
translation into accurate drawings.
%P%
The order of information directly matches the order of graphical
presentation in the drawn tree.  This means that in the "<generation>"
section, details of children are included directly beneath their
parents, and before aunts or uncles younger than their parents.
As shown in the example above, a different order is used in the "<ancestry>"
section, with an entire family given before the children of one 
member.  This ordering is convenient in the ancestry section because
it produces a more compact graphical layout.  I originally 
adopted the alternate ordering for ancestry, the same as used 
in the "<generation>" section,
which simplified writing a program for drawing the tree.  However, 
when I found its space usage to be excessive, I adopted the scheme
shown.
%L%%PH%%BD%
Program to Process a Tree Specification
%LT%%P%
Given the above design for digitally storing family tree data,
the provision of a program to output the tree(s) graphically
is not difficult.  Programs could be fairly easily written in
a number of languages and for a variety of types of hardware.
%P%
The approach adopted here is to write
the program in PostScript, making the output of drawings 
independent of the type of computer the tree is stored on,
and dependent only on the availability of a PostScript
printer in which the PostScript program will run, or a
suitable PostScript interpreter to convert the PostScript
instructions into a bitmap for output to a less intelligent
device.  
%P%
This approach has been previously adopted by the author in building
the Quikscript text processing system (Freeman, 1991, 1994), a 480 line program
with quite sophisticated facilities for placing text on the
page based on embedded layout instructions.  To handle the
combination of text and graphics that the family tree application
requires, the basic Quikscript program has been extended to
recognize the special keywords mentioned in the previous section.
This new program is entitled "GENEAL", and is 860
lines in length.
%P%
The intent of this design is for the person typing the tree into
the computer to send GENEAL and the tree to the PostScript printer
as a single file.  The tree is processed by GENEAL within the
printer and graphically output.  It is feasible for the tree
to be broken into sections in separate files, and for a single file
to be processed by GENEAL, provided that the break between sections
of the tree comes at the start of an "<ancestry>" section.
%P%
The advantages of this approach are portability of the program,
and compactness.  PostScript is seldom appreciated as a 
powerful high-level programming language, largely because of
its difficult-to-read syntax and poor debugging facilities.
The program does not take long to transfer to the printer, and is
quite fast in execution.
%P%
One disadvantage of this approach is the absence of diagnostic
facilities when the execution of the program is taking place within
the printer itself.  Errors in tree specification manifest themselves
in visible layout errors, rather than as warning messages.  
%P%
Another
disadvantage is that layout of the tree is based on built-in
heuristics, since the information is placed on the page as it is
read, rather than being stored for some optimization scheme to 
adopt a layout that takes into account the extent of information
to be presented.  The format of the family tree data does not
preclude a more sophisticated approach to layout; however, the
program described here demonstrates that the data can be handled
without a great deal of programming effort with quite satisfactory
results.
%L%%PH%%BD%
Index to the Tree
%LT%%P%
Locating individuals in a large family tree is generally problematic,
relying on searching over the sheet for a name familiar to the reader.  
Duplication of names in different generations and branches of the
family can make this more difficult.
When we split the tree across many pages, we only exacerbate the problem.
There is thus a need for some indexing scheme to enable the correct
page to be found more quickly.  
%P%
Date and place of birth provide a nearly unique way of identifying a 
person with a given name, and is the scheme used in maintaining
birth records and by the Church of Jesus Christ of Latter-day Saints
in maintaining their huge genealogy library.  However, family trees
built from anecdotal sources and with little recourse to formal 
birth, marriage and death certificates may not have much of this 
information.
%P%
The scheme I propose is to base the index on the ancestry detail,
which is almost always unique, unless someone has chosen to give
two children the same name.
All descendents on a sheet will be listed in the index under the
line of descent of the first ancestry section at the top of the page.
All comment and marriage detail is stripped out, and only direct
descendents of the first named person in the marriage are shown.
%P%
An example of this, based on part of the tree shown in Figure 1, 
could be:
%LT%%SZ,9%%P,30,-25,-0.7%%CW,90%
Lamont, Rhonda Kay     %TR,30%Frederich Kuffner / Adam Kuffner / Amelia Kuffner / Thomas Spence
%P%Lamont, Rosalyn Dawn     %TR,30%Frederich Kuffner / Adam Kuffner / Amelia Kuffner / Thomas Spence
%P%Shafer, Brendon     %TR,30%Frederich Kuffner / Adam Kuffner / Amelia Kuffner / Thomas Spence
%P%Shafer, Eric     %TR,30%Frederich Kuffner / Adam Kuffner / Amelia Kuffner / Thomas Spence / Darrin Shafer

%CW%%SZ%
%P,0,0,0%
The intent of this design is that when a tree is split across many pages,
a person should be recognizable by his or her name
and parentage, even if there are several people with the same name.
The ancestry is the overriding organizing principle of the trees,
and the correct page can be found by first finding the first name
given in the index (the oldest ancestor), and then the descendents 
in order in the following sheets,
until the correct page is found.  In this way, simple page numbers
are avoided, since subsequent additions require renumbering of 
all following pages, and would greatly complicate
maintenance of the index.  In this example,
the first few names are all on the one sheet, as shown in 
Figure 1, whereas the last name is on another sheet not shown,
distinguished by the last name in the ancestry.
%P%
I have written a short C program to produce the index information from a
collection of tree files.  The output is in a format compatible with
the family tree program, so it can be output on a PostScript printer
using GENEAL in the same way as the family tree files.  The index 
program requires a C compiler to compile, but is portable within
this restriction.  It assumes that all names in the tree are given
with surname as the last name component, though it will take care of
nicknames in parentheses, whether before or after the surname.
%L%%PH%%BD%
Conclusion
%LT%%P%
The ideas and programs discussed here have been tested on a family
tree comprising more than 700 descendents in six generations
and found to be workable.  Precision is
needed in preparing the electronic tree specifications of the families,
but errors are easily detected by inspecting the drawn trees and the
index file.
%P%
The amount of information that can be included in the "generation"
section of a page depends on the size of a family.  In Figure 1, children
and grandchildren are included, which can be typical for small families.
When there are many children, or there is a large amount of biographical
detail to be included, only the one family can fit onto one
page, increasing the need for an index.  If there is a very long ancestry 
section, the amount of space on a page for "generation" information
can become limited, and it could be forced over the page.
%P%
The programs are available by anonymous ftp from %IT%ftp.adfa.oz.au%RO% in the
directory %IT%pub/postscript%RO%.
%L%%PH%
%BD%References%LT%
%P,10,-10,-.5%
Freeman, T.G. (1991)  "QUIKSCRIPT:  A Portable Typesetter for PostScript 
Devices", %IT%Computer Science Department Report CS91/5%RO%, 
Australian Defence Force Academy.
%P%
Freeman, T.G. (1991)  "A Simple Portable Typesetting System",
%IT%Computer Science Department Report CS91/6%RO%, Australian 
Defence Force Academy.
%P%
Freeman, T.G. (1994)  "Guide to Quikscript". 
%IT%Computer Science Department Report CS05/94%RO%, Australian 
Defence Force Academy.
